// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "ProLink.h"
#include "MainFrm.h"

#include "rlManagerModulesDlg.h"

#include "DelimitedTextFile.h"
#include "DelimitedTextFileImportDlg.h"
#include "Table.h"

#include "TransferProgressDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CMainFrame

IMPLEMENT_DYNAMIC (CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP (CMainFrame, CFrameWnd)
	ON_WM_CREATE ()
	ON_WM_SETFOCUS ()
	ON_COMMAND (ID_FILE_NEW, OnFileNew)
	ON_COMMAND (ID_TRANSFER_MODULES, OnTransferModules)
	ON_NOTIFY (LVN_GETDISPINFO, AFX_IDW_PANE_FIRST, OnGetDispInfo)
	ON_NOTIFY (LVN_ENDLABELEDIT, AFX_IDW_PANE_FIRST, OnEndLabelEdit)
	ON_COMMAND(ID_TRANSFER_UPLOADALL, OnTransferUploadAll)
	ON_COMMAND(ID_TRANSFER_UPLOADSELECTED, OnTransferUploadSelected)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
	ON_COMMAND(ID_VIEW_DEBUGMESSAGES, OnViewDebugMessages)
	ON_COMMAND(ID_EDIT_DELETE, OnEditDelete)
	ON_UPDATE_COMMAND_UI(ID_EDIT_DELETE, OnUpdateUIItemSelected)
	ON_UPDATE_COMMAND_UI(ID_TRANSFER_UPLOADSELECTED, OnUpdateUIItemSelected)
END_MESSAGE_MAP ()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};


// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_pTransferProgressDialog = NULL;

	BOOL bFirstRun = AfxGetApp ()->GetProfileInt (_T(""), _T("First Run"), TRUE);

	if (bFirstRun)
	{
		OnTransferModules ();
		AfxGetApp ()->WriteProfileInt (_T(""), _T("First Run"), FALSE);
	}

}

CMainFrame::~CMainFrame()
{
	if (m_pTransferProgressDialog != NULL)
	{
		m_pTransferProgressDialog->DestroyWindow ();
		delete m_pTransferProgressDialog;
	}
}


int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1) return -1;

	// create the channel list view
	if (!m_wndChannelList.Create (AFX_WS_DEFAULT_VIEW | LVS_OWNERDATA, CRect (0, 0, 0, 0), this, AFX_IDW_PANE_FIRST))
	{
		TRACE0 ("Failed to create view window\n");
		return -1;
	}
	
	// create the main tool bar
	if (!m_wndToolBar.CreateEx (this, TBSTYLE_FLAT | TBSTYLE_TRANSPARENT) || !m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0 ("Failed to create toolbar\n");
		return -1;      // fail to create
	}
	// enable tooltips and status text
	m_wndToolBar.SetBarStyle (m_wndToolBar.GetBarStyle () | CBRS_TOOLTIPS | CBRS_FLYBY);

	/*
	// create the modules dialog bar
	if (!m_dlgModulesBar.Create (this, IDD_BAR_MODULES, CBRS_ALIGN_TOP, AFX_IDW_DIALOGBAR))
	{
		TRACE0 ("Failed to create Modules Bar\n");
		return -1;		// fail to create
	}

	// create the banks dialog bar
	if (!m_dlgBanksBar.Create (this, IDD_BAR_BANKS, CBRS_ALIGN_TOP, AFX_IDW_DIALOGBAR))
	{
		TRACE0 ("Failed to create Banks Bar\n");
		return -1;		// fail to create
	}
	*/

	// create the rebar control and add the tool bars and dialog bars to it
	//if (!m_wndReBar.Create(this) || !m_wndReBar.AddBar (&m_wndToolBar) || !m_wndReBar.AddBar (&m_dlgModulesBar) || !m_wndReBar.AddBar (&m_dlgBanksBar))
	if (!m_wndReBar.Create(this) || !m_wndReBar.AddBar (&m_wndToolBar))
	{
		TRACE0 ("Failed to create rebar\n");
		return -1;      // fail to create
	}

	/*
	// arrange the bands of the rebar control
	m_wndReBar.GetReBarCtrl ().MinimizeBand (0);
	m_wndReBar.GetReBarCtrl ().MinimizeBand (1);
	m_wndReBar.GetReBarCtrl ().MaximizeBand (2);
	*/

	// create the status bar and set the indicators
	if (!m_wndStatusBar.Create (this) || !m_wndStatusBar.SetIndicators (indicators, sizeof(indicators) / sizeof (UINT)))
	{
		TRACE0 ("Failed to create status bar\n");
		return -1;      // fail to create
	}

	/*
	m_dlgBanksBar.SendDlgItemMessage (IDC_BANKS, CB_ADDSTRING, 0, (LPARAM) &_T("(All Banks)"));
	m_dlgBanksBar.SendDlgItemMessage (IDC_BANKS, CB_SETCURSEL, 0, 0);

	m_dlgModulesBar.SendDlgItemMessage (IDC_MODULES, CB_ADDSTRING, 0, (LPARAM) &_T("(Default)"));
	m_dlgModulesBar.SendDlgItemMessage (IDC_MODULES, CB_SETCURSEL, 0, 0);
	*/

	m_wndChannelList.InsertColumn (CHANNEL_LIST_COLUMN_NUMBER, _T("#"), LVCFMT_CENTER, 30, CHANNEL_LIST_COLUMN_NUMBER);

	// for some reason, InsertColumn doesn't set the format on the first column, so we have to set it again ourselves
	LVCOLUMN ColumnInfo;
	ColumnInfo.mask = LVCF_FMT;
	ColumnInfo.fmt = LVCFMT_CENTER;
	m_wndChannelList.SetColumn (CHANNEL_LIST_COLUMN_NUMBER, &ColumnInfo);

	m_wndChannelList.InsertColumn (CHANNEL_LIST_COLUMN_FREQUENCY, _T("Frequency"), LVCFMT_CENTER, 100, CHANNEL_LIST_COLUMN_FREQUENCY);
	m_wndChannelList.InsertColumn (CHANNEL_LIST_COLUMN_DESCRIPTION, _T("Description"), LVCFMT_LEFT, 300, CHANNEL_LIST_COLUMN_DESCRIPTION);
	m_wndChannelList.InsertColumn (CHANNEL_LIST_COLUMN_CARNUMBER, _T("Car Number"), LVCFMT_CENTER, 75, CHANNEL_LIST_COLUMN_CARNUMBER);
	m_wndChannelList.InsertColumn (CHANNEL_LIST_COLUMN_DELAY, _T("Delay"), LVCFMT_CENTER, 75, CHANNEL_LIST_COLUMN_DELAY);
	m_wndChannelList.InsertColumn (CHANNEL_LIST_COLUMN_LOCKOUT, _T("Lockout"), LVCFMT_CENTER, 75, CHANNEL_LIST_COLUMN_LOCKOUT);
	m_wndChannelList.InsertColumn (CHANNEL_LIST_COLUMN_PRIORITY, _T("Priority"), LVCFMT_CENTER, 75, CHANNEL_LIST_COLUMN_PRIORITY);

	OnFileNew ();

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	cs.lpszClass = ::AfxRegisterWndClass (0, 0, 0, AfxGetApp ()->LoadIcon (IDR_MAINFRAME));

	return TRUE;
}


// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

const CString CMainFrame::GetItemString (const int nItem, const int nSubItem)
{
	rlMemoryChannel *pMemoryChannel = m_MemoryChannelArray.GetAt (nItem);
	CString strTemp;

	switch (nSubItem)
	{
		case CHANNEL_LIST_COLUMN_NUMBER:
			strTemp.Format ("%d", pMemoryChannel->GetChannelNumber ());
			break;
		case CHANNEL_LIST_COLUMN_FREQUENCY:
			if (pMemoryChannel->GetFrequency ().GetFrequencyMHz () != 0)
			{
				strTemp.Format ("%.4f", pMemoryChannel->GetFrequency ().GetFrequencyMHz ());
			}
			break;
		case CHANNEL_LIST_COLUMN_DESCRIPTION:
			strTemp = pMemoryChannel->GetDescription ();
			break;
		case CHANNEL_LIST_COLUMN_CARNUMBER:
			strTemp = pMemoryChannel->GetCarNumber ();
			break;
		case CHANNEL_LIST_COLUMN_DELAY:
			strTemp = pMemoryChannel->GetDelay ()?_T("True"):_T("False");
			break;
		case CHANNEL_LIST_COLUMN_LOCKOUT:
			strTemp = pMemoryChannel->GetLockout ()?_T("True"):_T("False");
			break;
		case CHANNEL_LIST_COLUMN_PRIORITY:
			strTemp = pMemoryChannel->GetPriority ()?_T("True"):_T("False");
			break;
		default:
			break;
	};

	return strTemp;
}

void CMainFrame::SetItemString (const int nItem, const int nSubItem, const CString strText)
{
	rlMemoryChannel *pMemoryChannel = m_MemoryChannelArray.GetAt (nItem);
	int nChannelNumber;
	rlFrequency Frequency;

	switch (nSubItem)
	{
		case CHANNEL_LIST_COLUMN_NUMBER:
			nChannelNumber = _tstoi (strText);
			pMemoryChannel->SetChannelNumber (nChannelNumber);
			break;
		case CHANNEL_LIST_COLUMN_FREQUENCY:
			Frequency.SetFrequencyMHz (strtod (strText, NULL));
			pMemoryChannel->SetFrequency (&Frequency);
			break;
		case CHANNEL_LIST_COLUMN_DESCRIPTION:
			pMemoryChannel->SetDescription (strText);
			break;
		case CHANNEL_LIST_COLUMN_CARNUMBER:
			pMemoryChannel->SetCarNumber (strText);
			break;
		case CHANNEL_LIST_COLUMN_DELAY:
			if (!strText.CompareNoCase (_T("true")) || !strText.CompareNoCase (_T("yes")) || !strText.CompareNoCase (_T("On")))
			{
				pMemoryChannel->SetDelay (true);
			}
			else
			{
				pMemoryChannel->SetDelay (false);
			}
			break;
		case CHANNEL_LIST_COLUMN_LOCKOUT:
			if (!strText.CompareNoCase (_T("true")) || !strText.CompareNoCase (_T("yes")) || !strText.CompareNoCase (_T("On")))
			{
				pMemoryChannel->SetLockout (true);
			}
			else
			{
				pMemoryChannel->SetLockout (false);
			}
			break;
		case CHANNEL_LIST_COLUMN_PRIORITY:
			if (!strText.CompareNoCase (_T("true")) || !strText.CompareNoCase (_T("yes")) || !strText.CompareNoCase (_T("On")))
			{
				pMemoryChannel->SetPriority (true);
			}
			else
			{
				pMemoryChannel->SetPriority (false);
			}
			break;
		default:
			break;
	};
}

// CMainFrame message handlers

void CMainFrame::OnSetFocus(CWnd* /*pOldWnd*/)
{
	// forward focus to the view window
	m_wndChannelList.SetFocus();
}

BOOL CMainFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	// let the view have first crack at the command
	if (m_wndChannelList.OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	// otherwise, do default handling
	return CFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CMainFrame::OnSetRange (const int nLower, const int nUpper)
{
	if (m_pTransferProgressDialog != NULL)
	{
		m_pTransferProgressDialog->m_ctrlProgress.SetRange32 (nLower, nUpper);
	}
}

void CMainFrame::OnSetPosition (const int nPosition)
{
	if (m_pTransferProgressDialog != NULL)
	{
		m_pTransferProgressDialog->m_ctrlProgress.SetPos (nPosition);
	}
}

afx_msg void CMainFrame::OnGetDispInfo (NMHDR *pNotifyStruct, LRESULT * /*lResult*/)
{
	NMLVDISPINFO *pDispInfo = (NMLVDISPINFO *) pNotifyStruct;
	LVITEM *pItem = &(pDispInfo)->item;

	if (pItem->mask & LVIF_TEXT) // need to fill in text
	{
		int nItem = pItem->iItem;
		int nSubItem = pItem->iSubItem;

		CString strTemp = GetItemString (nItem, nSubItem);
		_tcscpy (pItem->pszText, strTemp);
	}
}

afx_msg void CMainFrame::OnEndLabelEdit (NMHDR *pNotifyStruct, LRESULT *lResult)
{
	NMLVDISPINFO *pDispInfo = (NMLVDISPINFO *) pNotifyStruct;
	LVITEM *pItem = &(pDispInfo)->item;
	int nItem = pItem->iItem;
	int nSubItem = pItem->iSubItem;
	CString strText = pItem->pszText;

	SetItemString (nItem, nSubItem, strText);

	*lResult = 0;
}

void CMainFrame::OnFileNew()
{
	rlModule *pModule;
	m_RadioLinkManager.GetDefaultModule (&pModule);
	rlMemoryLayout *pMemoryLayout;
	pModule->GetMemoryLayout (&pMemoryLayout);

	int nFirstChannel = pMemoryLayout->GetFirstChannel ();
	int nLastChannel = pMemoryLayout->GetLastChannel ();

	m_RadioLinkManager.ReleaseModule (pModule);

	m_strFileName.Empty ();

	m_wndChannelList.SetRedraw (FALSE);
	m_wndChannelList.DeleteAllItems ();
	m_MemoryChannelArray.Empty ();

	CString strTemp;
	rlMemoryChannel *pChannelTemp;
	for (int x = nFirstChannel; x <= nLastChannel; x++)
	{
		strTemp.Format ("%d", x);
		m_wndChannelList.InsertItem (x, NULL);
		m_MemoryChannelArray.Add (&pChannelTemp);
		pChannelTemp->SetChannelNumber (x);
	}

	m_wndChannelList.SetRedraw (TRUE);
}

void CMainFrame::OnTransferModules ()
{
	rlManagerModulesDlg dlg;
	dlg.DoModal ();
}

void CMainFrame::OnTransferUploadAll ()
{
	rlModule *pModule;
	m_RadioLinkManager.GetDefaultModule (&pModule);

	rlMemory *pMemory;
	HRESULT hResult = pModule->GetMemory (&pMemory);
	if (FAILED (hResult))
	{
		CString strErrorMessage;
		strErrorMessage.Format (_T("An error occured while attempting to retrieve the rlMemory interface from the selected RadioLink Module.\n\nrlModule::GetMemory () returned error code 0x%X.\n\nPlease contact the maker of this RadioLink Module for further assistance."), hResult);
		::AfxMessageBox (strErrorMessage);
		m_RadioLinkManager.ReleaseModule (pModule);
		return;
	}

	if (m_pTransferProgressDialog == NULL) delete m_pTransferProgressDialog;
	m_pTransferProgressDialog = new CTransferProgressDialog (this);
	m_pTransferProgressDialog->Create (IDD_TRANSFER_PROGRESS, this);

	pMemory->SetProgressTarget (this);

	hResult = pMemory->SetAllChannels (&m_MemoryChannelArray);
	if (FAILED (hResult))
	{
		GDDEBUGHRESULT (3, "rlMemory::SetAllChennels ()", hResult);
		::AfxMessageBox (_T("A transfer error has occurred."), MB_ICONEXCLAMATION | MB_OK);
	}

	m_pTransferProgressDialog->DestroyWindow ();
	delete m_pTransferProgressDialog;
	m_pTransferProgressDialog = NULL;

	m_RadioLinkManager.ReleaseModule (pModule);
}

void CMainFrame::OnTransferUploadSelected()
{
	int nSelectedCount = m_wndChannelList.GetSelectedCount ();
	if (nSelectedCount == 0)
	{
		return;
	}

	rlModule *pModule;
	m_RadioLinkManager.GetDefaultModule (&pModule);

	rlMemory *pMemory;
	HRESULT hResult = pModule->GetMemory (&pMemory);
	if (FAILED (hResult))
	{
		GDDEBUGHRESULT (3, "rlModule::GetMemory ()", hResult);

		CString strErrorMessage;
		strErrorMessage.Format (_T("An error occured while attempting to retrieve the rlMemory interface from the selected RadioLink Module.\n\nrlModule::GetMemory () returned error code 0x%X.\n\nPlease contact the maker of this RadioLink Module for further assistance."), hResult);
		::AfxMessageBox (strErrorMessage);
		m_RadioLinkManager.ReleaseModule (pModule);
		return;
	}

	if (m_pTransferProgressDialog == NULL) delete m_pTransferProgressDialog;
	m_pTransferProgressDialog = new CTransferProgressDialog (this);
	m_pTransferProgressDialog->Create (IDD_TRANSFER_PROGRESS, this);

	m_pTransferProgressDialog->m_ctrlProgress.SetRange (0, (short) nSelectedCount);

	pMemory->SetChannelMultiStart (nSelectedCount);

	int nRow = m_wndChannelList.GetNextItem (-1, LVNI_SELECTED);
	int nProgress = 0;

	while (nRow != -1)
	{
		m_pTransferProgressDialog->m_ctrlProgress.SetPos (nProgress);

		if (!SUCCEEDED (pMemory->SetChannelMulti (m_MemoryChannelArray.GetAt (nRow))))
		{
			::AfxMessageBox (_T("A transfer error has occurred."), MB_ICONEXCLAMATION | MB_OK);
		}

		nRow = m_wndChannelList.GetNextItem (nRow, LVNI_SELECTED);
		nProgress++;
	}

	pMemory->SetChannelMultiEnd ();

	m_pTransferProgressDialog->DestroyWindow ();
	delete m_pTransferProgressDialog;
	m_pTransferProgressDialog = NULL;

	m_RadioLinkManager.ReleaseModule (pModule);
}

void CMainFrame::OnFileOpen()
{
	CFileDialog dlgOpen (TRUE, "csv", NULL, OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, _T("Delimited Text File (*.csv, *.txt)|*.csv;*.txt|All Files (*.*)|*.*||"), this);

	if (dlgOpen.DoModal () != IDOK) return;

	m_strFileName = dlgOpen.GetPathName ();

	CTable ChannelDataTable;
	CDelimitedTextFile dtfTextFile (&ChannelDataTable);
	if (!dtfTextFile.LoadFile (&m_strFileName))
	{
		::AfxMessageBox (_T("The file could not be opened.\n"), MB_ICONEXCLAMATION | MB_OK);
	}

	//CDelimitedTextFileImportDlg dlgImport;
	//dlgImport.SetDataTable (&ChannelDataTable);
	//dlgImport.DoModal ();

	m_MemoryChannelArray.Empty ();
	m_wndChannelList.SetRedraw (FALSE);
	m_wndChannelList.DeleteAllItems ();

	rlMemoryChannel *pChannelTemp;
	CString strTemp;

	for (int nRow = 0; nRow < ChannelDataTable.GetRowCount () - 1; nRow++)
	{
		m_MemoryChannelArray.Add (&pChannelTemp);
		//pChannelTemp->SetChannelNumber (nRow + 1);

		m_wndChannelList.InsertItem (nRow, NULL);

		for (int nColumn = 0; nColumn < ChannelDataTable.GetColumnCount (); nColumn++)
		{
			strTemp = ChannelDataTable.GetCell (nRow + 1, nColumn)->GetDataString ();
			SetItemString (nRow, nColumn, strTemp);
		}
	}

	m_wndChannelList.SetRedraw (TRUE);
}

void CMainFrame::OnFileSave()
{
	if (m_strFileName.IsEmpty ())
	{
		OnFileSaveAs ();
		return;
	}

	CTable ChannelDataTable;

	ChannelDataTable.AddColumn (7);
	ChannelDataTable.AddRow (m_wndChannelList.GetItemCount () + 1);

	CTableCell cellID (_T("ID"));
	CTableCell cellFrequency (_T("Frequency"));
	CTableCell cellDescription (_T("Description"));
	CTableCell cellNumber (_T("Car Number"));
	CTableCell cellDelay (_T("Delay"));
	CTableCell cellLockout (_T("Lockout"));
	CTableCell cellPriority (_T("Priority"));

	ChannelDataTable.SetCell (0, CHANNEL_LIST_COLUMN_NUMBER, &cellID);
	ChannelDataTable.SetCell (0, CHANNEL_LIST_COLUMN_FREQUENCY, &cellFrequency);
	ChannelDataTable.SetCell (0, CHANNEL_LIST_COLUMN_DESCRIPTION, &cellDescription);
	ChannelDataTable.SetCell (0, CHANNEL_LIST_COLUMN_CARNUMBER, &cellNumber);
	ChannelDataTable.SetCell (0, CHANNEL_LIST_COLUMN_DELAY, &cellDelay);
	ChannelDataTable.SetCell (0, CHANNEL_LIST_COLUMN_LOCKOUT, &cellLockout);
	ChannelDataTable.SetCell (0, CHANNEL_LIST_COLUMN_PRIORITY, &cellPriority);

	CString strItemString;
	CTableCell CellData;

	for (int nRow = 0; nRow < m_wndChannelList.GetItemCount (); nRow++)
	{
		for (int nColumn = 0; nColumn < ChannelDataTable.GetColumnCount (); nColumn++)
		{
			strItemString = GetItemString (nRow, nColumn);

			if (nColumn == CHANNEL_LIST_COLUMN_CARNUMBER)
			{
				CellData.SetData (&strItemString);
			}
			else
			{
				CellData.SetDataAutoType (&strItemString);
			}
			
			ChannelDataTable.SetCell (nRow + 1, nColumn, &CellData);
		}
	}

	CDelimitedTextFile dtfTextFile (&ChannelDataTable);
	if (!dtfTextFile.SaveFile (&m_strFileName))
	{
		::AfxMessageBox (_T("The file could not be saved.\n"), MB_ICONEXCLAMATION | MB_OK);
	}
}

void CMainFrame::OnFileSaveAs()
{
	CFileDialog dlgSaveAs (FALSE, "csv", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, _T("Delimited Text File (*.csv, *.txt)|*.csv;*.txt|All Files (*.*)|*.*||"), this);
	
	if (dlgSaveAs.DoModal () != IDOK) return;

	m_strFileName = dlgSaveAs.GetPathName ();

	OnFileSave ();
}

void CMainFrame::OnViewDebugMessages()
{
	gdGetDebug ()->ShowDebugMessagesDialog (this);
}

void CMainFrame::OnEditDelete()
{
	m_wndChannelList.SetRedraw (FALSE);

	rlMemoryChannel *pMemoryChannel;

	for (int nItem = 0; nItem < m_wndChannelList.GetItemCount (); nItem++)
	{
		if (m_wndChannelList.GetItemState (nItem, LVIS_SELECTED))
		{
			pMemoryChannel = m_MemoryChannelArray.GetAt (nItem);

			pMemoryChannel->Empty ();
			pMemoryChannel->SetChannelNumber (nItem + 1);
		}
	}

	m_wndChannelList.SetRedraw (TRUE);
	m_wndChannelList.Invalidate ();
}

void CMainFrame::OnUpdateUIItemSelected (CCmdUI *pCmdUI)
{
	bool bItemSelected = false;

	for (int nItem = 0; nItem < m_wndChannelList.GetItemCount (); nItem++)
	{
		if (m_wndChannelList.GetItemState (nItem, LVIS_SELECTED))
		{
			bItemSelected = true;
			break;
		}
	}

	if (bItemSelected)
	{
		pCmdUI->Enable (TRUE);
	}
	else
	{
		pCmdUI->Enable (FALSE);
	}
}
