#include "StdAfx.h"
#include "gddebug.h"

#include "gdDebugMessagesDialog.h"

gdDebug G_GD_DEBUG_OBJECT;

inline gdDebug *gdGetDebug (void)
{
	return &G_GD_DEBUG_OBJECT;
}

gdDebug::gdDebug(void)
{
	m_nVerbosity = 0; // default to verbose mode
	m_pMessagesDialog = new gdDebugMessagesDialog;
}

gdDebug::~gdDebug(void)
{
	delete m_pMessagesDialog;
}

void gdDebug::SetVerbosity (const int nVerbosity)
{
	GDDEBUG (0, "Verbosity level changed to %d.", nVerbosity);
	m_nVerbosity = nVerbosity;
}

void gdDebug::Message (const int nSeverity, LPCTSTR lpMessage, ...)
{
	if (m_nVerbosity > nSeverity) return; // ignore messages less than the specified verbosity

	va_list args;
	va_start (args, lpMessage);

	void *pArgs = *(void **)args;

	CString strTemp;
	strTemp.Format (lpMessage, pArgs);

	if (IsWindow (*m_pMessagesDialog)) m_pMessagesDialog->AddMessage (strTemp);

	strTemp += '\n';
	TRACE (strTemp);

	va_end (args);
}

void gdDebug::MessageBinary (const int nSeverity, const DWORD dwLength, const BYTE *lpBuffer)
{
	const int cnMaxLineLength = 16;
	int nLineLength = 0;
	BYTE cTemp;
	CString strTemp;
	CString strLine;
	CString strLineHex;
	CString strLineAscii;

	for (DWORD x = 0; x < dwLength; x++)
	{
		nLineLength++;
		cTemp = lpBuffer [x];

		strTemp.Format ("%02X", cTemp);
		strLineHex += ' ' + strTemp;

		if ((cTemp < 32) || (cTemp >= 127)) cTemp = '?';
		strTemp.Format ("%c", cTemp);
		strLineAscii += strTemp;

		if (nLineLength >= cnMaxLineLength)
		{
			strLine.Format ("%s - %s", strLineHex, strLineAscii);
			Message (nSeverity, strLine);
			strLineHex.Empty ();
			strLineAscii.Empty ();
			nLineLength = 0;
		}
	}

	for (x = nLineLength; x < cnMaxLineLength; x++)
	{
		strLineHex += "   ";
	}

	strLine.Format ("%s - %s", strLineHex, strLineAscii);
	Message (nSeverity, strLine);
}

void gdDebug::MessageResultCode (const int nSeverity, const LPCTSTR lpBuffer, const HRESULT hResult)
{
	LPVOID lpMessageBuffer;
	DWORD dwLength = ::FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, NULL, hResult, MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpMessageBuffer, 0, NULL);

	((LPTSTR)lpMessageBuffer) [dwLength - 2] = '\0'; // remove newline from end

	CString strMessage;
	strMessage.Format (_T("%s returned error code 0x%08X (%s)"), lpBuffer, hResult, lpMessageBuffer);

	Message (nSeverity, strMessage);

	LocalFree (lpMessageBuffer);
}

void gdDebug::ShowDebugMessagesDialog (CWnd *pParent)
{
	if (!IsWindow (*m_pMessagesDialog))
	{
		m_pMessagesDialog->Create (IDD_DEBUG_MESSAGES, pParent);
	}
	
	m_pMessagesDialog->ShowWindow (SW_SHOWDEFAULT);
}

