// gdGridCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "ProLink.h"
#include "gdGridCtrl.h"

#include "gdGridCtrlEdit.h"

// gdGridCtrl

IMPLEMENT_DYNAMIC(gdGridCtrl, CListCtrl)
gdGridCtrl::gdGridCtrl ()
{
}

gdGridCtrl::~gdGridCtrl ()
{
}


BEGIN_MESSAGE_MAP (gdGridCtrl, CListCtrl)
	ON_WM_CHAR ()
	ON_WM_LBUTTONDBLCLK ()
	ON_WM_VSCROLL ()
	ON_WM_HSCROLL ()
	ON_WM_MOUSEWHEEL ()
	ON_NOTIFY_REFLECT_EX (LVN_ENDLABELEDIT, OnEndLabelEdit)
	ON_COMMAND(ID_EDIT_SELECTALL, OnEditSelectAll)
	ON_COMMAND_EX(ID_EDIT_DELETE, OnEditDelete)
END_MESSAGE_MAP ()



// gdGridCtrl message handlers

BOOL gdGridCtrl::Create (DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID)
{
	//dwStyle &= ~WS_BORDER;
	dwStyle |= LVS_REPORT;

	if (!CListCtrl::Create (dwStyle, rect, pParentWnd, nID)) return FALSE;

	SetExtendedStyle (GetExtendedStyle () | LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES | LVS_EX_HEADERDRAGDROP);

	return TRUE;
}

HRESULT gdGridCtrl::EditItem (const int nItem, const int nSubItem, CEdit **ppEditCtrl)
{
	SetFocus ();
	SelectItem (nItem, true);

	// TODO: fix this so that we can edit sub item 0
	if (nSubItem == 0) return NULL;

	if (!EnsureVisible (nItem, FALSE)) return NULL;

	CHeaderCtrl *pHeaderCtrl = (CHeaderCtrl *) GetDlgItem (0);
	int nColumnCount = pHeaderCtrl->GetItemCount ();

	if (nSubItem >= nColumnCount || GetColumnWidth (nSubItem) < 5) return NULL;

	CRect rcSubItem;
	GetSubItemRect (nItem, nSubItem, LVIR_BOUNDS, rcSubItem);

	CRect rcClient;
	GetClientRect (&rcClient);

	if (rcSubItem.left < rcClient.left || rcSubItem.right > rcClient.right)
	{
		CSize Size;
		if (rcSubItem.left < rcClient.left)
			Size.cx = rcSubItem.left - rcClient.left;
		else if (rcSubItem.right > rcClient.right)
			Size.cx = rcSubItem.right - rcClient.right;

		Size.cy = 0;

		Scroll (Size);

		GetSubItemRect (nItem, nSubItem, LVIR_BOUNDS, rcSubItem);
	}

	LV_COLUMN ColumnInfo;
	ColumnInfo.mask = LVCF_FMT;
	GetColumn (nSubItem, &ColumnInfo);

	int nColumnJustification = ColumnInfo.fmt & LVCFMT_JUSTIFYMASK;

	DWORD dwStyle;
	if (nColumnJustification == LVCFMT_LEFT)
	{
		dwStyle = ES_LEFT;
	}
	else if (nColumnJustification == LVCFMT_RIGHT)
	{
		dwStyle = ES_RIGHT;
	}
	else
	{
		dwStyle = ES_CENTER;
	}

	rcSubItem.top -= 3;
	if (rcSubItem.right > rcClient.right) rcSubItem.right = rcClient.right;

	dwStyle |= WS_BORDER | WS_CHILD | WS_VISIBLE | ES_AUTOHSCROLL;

	CEdit *pEditCtrl = DEBUG_NEW gdGridCtrlEdit (this, nItem, nSubItem, GetItemText (nItem, nSubItem));
	pEditCtrl->Create (dwStyle, rcSubItem, this, 0);

	if (ppEditCtrl != NULL) *ppEditCtrl = pEditCtrl;

	return S_OK;
}

const int gdGridCtrl::GetNextItemUp (const int nItem)
{
	int nNextItem = nItem - 1;

	if (nNextItem < 0) nNextItem = 0;

	return nNextItem;
}

const int gdGridCtrl::GetNextItemDown (const int nItem, const bool /*bCreate*/)
{
	int nNextItem = nItem + 1;

	if (nNextItem > GetItemCount ()) nNextItem = GetItemCount ();

	return nNextItem;
}

const int gdGridCtrl::GetNextSubItemLeft (const int nSubItem)
{
	int nCount = GetHeaderCtrl ()->GetItemCount ();
	LPINT pnOrder = DEBUG_NEW int [nCount];
	GetColumnOrderArray (pnOrder, nCount);

	int nNextItem = 0;

	for (int x = 0; x < nCount; x++)
	{
		if (pnOrder [x] == nSubItem)
		{
			nNextItem = x - 1;
			if (nNextItem < 0) nNextItem = 0;
			nNextItem = pnOrder [nNextItem];
			break;
		}
	}

	delete pnOrder;

	return nNextItem;
}

const int gdGridCtrl::GetNextSubItemRight (const int nSubItem)
{
	int nCount = GetHeaderCtrl ()->GetItemCount ();
	LPINT pnOrder = DEBUG_NEW int [nCount];
	GetColumnOrderArray (pnOrder, nCount);

	int nNextItem = 0;

	for (int x = 0; x < nCount; x++)
	{
		if (pnOrder [x] == nSubItem)
		{
			nNextItem = x + 1;
			if (nNextItem > nCount) nNextItem = nCount;
			nNextItem = pnOrder [nNextItem];
			break;
		}
	}

	delete pnOrder;

	return nNextItem;
}


void gdGridCtrl::OnChar (UINT nChar, UINT /*nRepCnt*/, UINT nFlags)
{
	if (GetSelectedCount () == 1)
	{
		int nItem = GetNextItem (-1, LVNI_SELECTED);

		CEdit *pEdit = NULL;
		if (FAILED (EditItem (nItem, 1, &pEdit))) return;

		pEdit->SendMessage (WM_CHAR, nChar, nFlags);
	}
}

void gdGridCtrl::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	CListCtrl::OnLButtonDblClk (nFlags, point);

	LVHITTESTINFO HitTestInfo;
	HitTestInfo.pt = point;
	SubItemHitTest (&HitTestInfo);

	if ((HitTestInfo.flags && LVHT_ONITEM) != 0)
	{
		if ((GetItemState (HitTestInfo.iItem, LVIS_FOCUSED) & LVIS_FOCUSED) == LVIS_FOCUSED)
		{
			EditItem (HitTestInfo.iItem, HitTestInfo.iSubItem, NULL);
		}
	}
}

void gdGridCtrl::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	if (GetFocus () != this) SetFocus ();
	
	CListCtrl::OnVScroll (nSBCode, nPos, pScrollBar);
}

void gdGridCtrl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	if (GetFocus () != this) SetFocus ();

	CListCtrl::OnHScroll (nSBCode, nPos, pScrollBar);
}

BOOL gdGridCtrl::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt)
{
	if (GetFocus () != this) SetFocus ();

	return CListCtrl::OnMouseWheel (nFlags, zDelta, pt);
}

BOOL gdGridCtrl::OnEndLabelEdit (NMHDR *pNMHDR, LRESULT *)
{
	if ((GetStyle() & LVS_OWNERDATA) != 0) return FALSE; // let the parent handle this message

	NMLVDISPINFO *pDispInfo = reinterpret_cast <NMLVDISPINFO *> (pNMHDR);
	LVITEM *pItem = &pDispInfo->item;

	if (pItem->pszText != NULL)
	{
		SetItemText (pItem->iItem, pItem->iSubItem, pItem->pszText);
	}

	return TRUE;
}

void gdGridCtrl::OnEditSelectAll()
{
	SetRedraw (FALSE);

	for (int nItem = 0; nItem < GetItemCount (); nItem++)
	{
		SetItemState (nItem, LVIS_SELECTED, LVIS_SELECTED);
	}

	SetRedraw (TRUE);
}

BOOL gdGridCtrl::OnEditDelete(UINT)
{
	if ((GetStyle() & LVS_OWNERDATA) != 0) return FALSE; // let the parent handle this message

	SetRedraw (FALSE);

	int nSubItem;
	LVCOLUMN Column;
	Column.mask = 0;

	for (int nItem = 0; nItem < GetItemCount (); nItem++)
	{
		if (GetItemState (nItem, LVIS_SELECTED))
		{
			for (nSubItem = 1; GetColumn (nSubItem, &Column); nSubItem++)
			{
				SetItemText (nItem, nSubItem, NULL);
			}
		}
	}

	SetRedraw (TRUE);

	return TRUE;
}

void gdGridCtrl::SelectItem(int nItem, bool bDeselectOthers)
{
	if (bDeselectOthers)
	{
		int nSelectedItem;
		POSITION pos = GetFirstSelectedItemPosition ();

		while (pos != NULL)
		{
			nSelectedItem = GetNextSelectedItem (pos);
			SetItemState (nSelectedItem, 0, LVIS_SELECTED);
		}
	}

	SetItemState (nItem, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);
}
