// ChannelListView.cpp : implementation file
//

#include "stdafx.h"
#include "ProLink.h"
#include "ChannelListView.h"

#include "ProLinkDocument.h"

#define ID_CHANNELGRIDCTRL	AFX_IDW_PANE_FIRST + 2

// CChannelListView

IMPLEMENT_DYNCREATE(CChannelListView, CView)

CChannelListView::CChannelListView()
{
}

CChannelListView::~CChannelListView()
{
}

BEGIN_MESSAGE_MAP(CChannelListView, CView)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)

	ON_WM_SIZE()
	ON_WM_CONTEXTMENU()
	ON_NOTIFY (LVN_GETDISPINFO, ID_CHANNELGRIDCTRL, OnGetDispInfo)
	ON_NOTIFY (LVN_ENDLABELEDIT, ID_CHANNELGRIDCTRL, OnEndLabelEdit)
END_MESSAGE_MAP()


// CChannelListView drawing

void CChannelListView::OnDraw (CDC* /*pDC*/)
{
	//CDocument *pDoc = GetDocument ();
}


// CChannelListView printing

BOOL CChannelListView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CChannelListView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CChannelListView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}


// CChannelListView diagnostics

#ifdef _DEBUG
void CChannelListView::AssertValid() const
{
	CView::AssertValid();
}

void CChannelListView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG


// CChannelListView message handlers

BOOL CChannelListView::Create (LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext)
{
	if (!CView::Create (lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext)) return FALSE;

	m_GridCtrl.Create (dwStyle | LVS_OWNERDATA, rect, this, ID_CHANNELGRIDCTRL);

	m_GridCtrl.InsertColumn (CHANNEL_LIST_COLUMN_NUMBER, _T("#"), LVCFMT_CENTER, 30, CHANNEL_LIST_COLUMN_NUMBER);

	// for some reason, InsertColumn doesn't set the format on the first column, so we have to set it again ourselves
	LVCOLUMN ColumnInfo;
	ColumnInfo.mask = LVCF_FMT;
	ColumnInfo.fmt = LVCFMT_CENTER;
	m_GridCtrl.SetColumn (CHANNEL_LIST_COLUMN_NUMBER, &ColumnInfo);

	m_GridCtrl.InsertColumn (CHANNEL_LIST_COLUMN_FREQUENCY, _T("Frequency"), LVCFMT_CENTER, 100, CHANNEL_LIST_COLUMN_FREQUENCY);
	m_GridCtrl.InsertColumn (CHANNEL_LIST_COLUMN_DESCRIPTION, _T("Description"), LVCFMT_LEFT, 300, CHANNEL_LIST_COLUMN_DESCRIPTION);
	m_GridCtrl.InsertColumn (CHANNEL_LIST_COLUMN_CARNUMBER, _T("Car Number"), LVCFMT_CENTER, 75, CHANNEL_LIST_COLUMN_CARNUMBER);
	m_GridCtrl.InsertColumn (CHANNEL_LIST_COLUMN_DELAY, _T("Delay"), LVCFMT_CENTER, 75, CHANNEL_LIST_COLUMN_DELAY);
	m_GridCtrl.InsertColumn (CHANNEL_LIST_COLUMN_LOCKOUT, _T("Lockout"), LVCFMT_CENTER, 75, CHANNEL_LIST_COLUMN_LOCKOUT);
	m_GridCtrl.InsertColumn (CHANNEL_LIST_COLUMN_PRIORITY, _T("Priority"), LVCFMT_CENTER, 75, CHANNEL_LIST_COLUMN_PRIORITY);

	m_GridCtrl.InsertItem (0, NULL);

	return TRUE;
}

void CChannelListView::OnSize (UINT nType, int cx, int cy)
{
	CView::OnSize (nType, cx, cy);

	// TODO: fix this kludge
	if (cx || cy) m_GridCtrl.MoveWindow (0, 0, cx, cy);
}

void CChannelListView::OnContextMenu(CWnd *pWnd, CPoint point)
{
	CMenu Menu;
	VERIFY (Menu.LoadMenu (IDR_CONTEXT_CHANNELLIST));

	CMenu *pContextMenu = Menu.GetSubMenu (0);
	ASSERT (pContextMenu != NULL);

	pContextMenu->TrackPopupMenu (TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, pWnd);
}

afx_msg void CChannelListView::OnGetDispInfo (NMHDR *pNotifyStruct, LRESULT * /*lResult*/)
{
	NMLVDISPINFO *pDispInfo = (NMLVDISPINFO *) pNotifyStruct;
	LVITEM *pItem = &(pDispInfo)->item;

	if (pItem->mask & LVIF_TEXT) // need to fill in text
	{
		int nItem = pItem->iItem;
		int nSubItem = pItem->iSubItem;

		CString strTemp = GetItemString (nItem, nSubItem);
		_tcscpy (pItem->pszText, strTemp);
	}
}

afx_msg void CChannelListView::OnEndLabelEdit (NMHDR *pNotifyStruct, LRESULT *lResult)
{
	NMLVDISPINFO *pDispInfo = (NMLVDISPINFO *) pNotifyStruct;
	LVITEM *pItem = &(pDispInfo)->item;
	int nItem = pItem->iItem;
	int nSubItem = pItem->iSubItem;
	CString strText = pItem->pszText;

	SetItemString (nItem, nSubItem, strText);
	m_pDocument->SetModifiedFlag ();

	*lResult = 0;
}

const CString CChannelListView::GetItemString (const int nItem, const int nSubItem)
{
	rlMemoryChannelArray *pChannelList = ((CProLinkDocument *) m_pDocument)->GetCurrentChannelList ();
	
	if (nItem >= pChannelList->GetCount ())
	{
		if (nSubItem == CHANNEL_LIST_COLUMN_NUMBER)
			return _T("*");
		else
			return NULL;
	}

	rlMemoryChannel *pMemoryChannel = pChannelList->GetAt (nItem);
	CString strTemp;

	switch (nSubItem)
	{
		case CHANNEL_LIST_COLUMN_NUMBER:
			strTemp.Format ("%d", pMemoryChannel->GetChannelNumber ());
			break;
		case CHANNEL_LIST_COLUMN_FREQUENCY:
			if (pMemoryChannel->GetFrequency ().GetFrequencyMHz () != 0)
			{
				strTemp.Format ("%.4f", pMemoryChannel->GetFrequency ().GetFrequencyMHz ());
			}
			break;
		case CHANNEL_LIST_COLUMN_DESCRIPTION:
			strTemp = pMemoryChannel->GetDescription ();
			break;
		case CHANNEL_LIST_COLUMN_CARNUMBER:
			strTemp = pMemoryChannel->GetCarNumber ();
			break;
		case CHANNEL_LIST_COLUMN_DELAY:
			strTemp = pMemoryChannel->GetDelay ()?_T("True"):_T("False");
			break;
		case CHANNEL_LIST_COLUMN_LOCKOUT:
			strTemp = pMemoryChannel->GetLockout ()?_T("True"):_T("False");
			break;
		case CHANNEL_LIST_COLUMN_PRIORITY:
			strTemp = pMemoryChannel->GetPriority ()?_T("True"):_T("False");
			break;
		default:
			break;
	};

	return strTemp;
}

void CChannelListView::SetItemString (const int nItem, const int nSubItem, const CString strText)
{
	rlMemoryChannelArray *pChannelList = ((CProLinkDocument *) m_pDocument)->GetCurrentChannelList ();
	rlMemoryChannel *pMemoryChannel;

	if (nItem >= pChannelList->GetCount ())
	{
		pChannelList->Add (&pMemoryChannel);
	}
	else
	{
        pMemoryChannel = pChannelList->GetAt (nItem);
	}

	rlFrequency Frequency;

	switch (nSubItem)
	{
		case CHANNEL_LIST_COLUMN_NUMBER:
			break;
		case CHANNEL_LIST_COLUMN_FREQUENCY:
			Frequency.SetFrequencyMHz (strtod (strText, NULL));
			pMemoryChannel->SetFrequency (&Frequency);
			break;
		case CHANNEL_LIST_COLUMN_DESCRIPTION:
			pMemoryChannel->SetDescription (strText);
			break;
		case CHANNEL_LIST_COLUMN_CARNUMBER:
			pMemoryChannel->SetCarNumber (strText);
			break;
		case CHANNEL_LIST_COLUMN_DELAY:
			if (!strText.CompareNoCase (_T("true")) || !strText.CompareNoCase (_T("yes")) || !strText.CompareNoCase (_T("On")))
			{
				pMemoryChannel->SetDelay (true);
			}
			else
			{
				pMemoryChannel->SetDelay (false);
			}
			break;
		case CHANNEL_LIST_COLUMN_LOCKOUT:
			if (!strText.CompareNoCase (_T("true")) || !strText.CompareNoCase (_T("yes")) || !strText.CompareNoCase (_T("On")))
			{
				pMemoryChannel->SetLockout (true);
			}
			else
			{
				pMemoryChannel->SetLockout (false);
			}
			break;
		case CHANNEL_LIST_COLUMN_PRIORITY:
			if (!strText.CompareNoCase (_T("true")) || !strText.CompareNoCase (_T("yes")) || !strText.CompareNoCase (_T("On")))
			{
				pMemoryChannel->SetPriority (true);
			}
			else
			{
				pMemoryChannel->SetPriority (false);
			}
			break;
		default:
			break;
	};

	m_pDocument->SetModifiedFlag ();
	m_pDocument->UpdateAllViews (NULL);
}

void CChannelListView::OnUpdate(CView* /*pSender*/, LPARAM /*lHint*/, CObject* /*pHint*/)
{
	rlMemoryChannelArray *pChannelList = ((CProLinkDocument *) m_pDocument)->GetCurrentChannelList ();
	if (pChannelList == NULL) return;

	int nItemCount = m_GridCtrl.GetItemCount ();
	int nRequiredCount = ((int) pChannelList->GetCount ()) + 1;

	if (nItemCount < nRequiredCount)
	{
		for (int x = nItemCount; x < nRequiredCount; x++)
		{
			m_GridCtrl.InsertItem (0, NULL);
		}
	}
}

void CChannelListView::OnPrint(CDC* pDC, CPrintInfo* /*pInfo*/)
{
	CRect rect (0, 0, 0, 0);

	pDC->DrawText ("Printing is not yet supported in ProLink.", &rect, DT_NOCLIP);
}
