// RadioLink.h

#pragma once


// Definitions
#define RL_MODULATION_UNKNOWN	0
#define RL_MODULATION_AUTO		1
#define RL_MODULATION_AM		2
#define RL_MODULATION_NFM		3
#define RL_MODULATION_FM		4
#define RL_MODULATION_WFM		5


// Forward Declarations
[export] __interface IModule;
[export] __interface IDeviceFeatures;
[export] __interface IMemory;
[export] __interface IMemoryLayout;
[export] __interface ISettings;
[export] __interface IControl;


// IModule
[
	object,
	uuid("DF626BB5-5889-4ea9-9AF2-A4C0661C9B09"),
	dual,
	helpstring("IModule Interface"),
	pointer_default(unique)
]
__interface IModule : IDispatch
{
	// Configuration Index
	[propget, id(1), helpstring("Gets and sets the configuration index of this instance of the module.")] HRESULT ConfigurationIndex([out, retval] USHORT* pVal);
	[propput, id(1), helpstring("Gets and sets the configuration index of this instance of the module.")] HRESULT ConfigurationIndex([in] USHORT newVal);

	// General Module Information
	[propget, id(2), helpstring("Returns the user-friendly name of the module.")] HRESULT ModuleName([out, retval] BSTR *pVal);
	[propget, id(3), helpstring("Returns the user-friendly description of the module.")] HRESULT ModuleDescription([out, retval] BSTR *pVal);

	// Device Features
	[propget, id(4), helpstring("Returns the devices's features interface.")] HRESULT Features([out, retval] IDeviceFeatures **pVal);

	// Supported Interfaces
	[propget, id(5), helpstring("Returns the module's memory interface.")] HRESULT Memory([out, retval] IMemory **pVal);
	[propget, id(6), helpstring("Returns the module's settings interface.")] HRESULT Settings([out, retval] ISettings **pVal);
	[propget, id(7), helpstring("Returns the module's control interface.")] HRESULT Control([out, retval] IControl **pVal);
};


// IDeviceFeatures
[
	object,
	uuid("65E5639B-08E5-4761-9569-F4043E54D0AC"),
	dual,
	helpstring("IDeviceFeatures Interface"),
	pointer_default(unique)
]
__interface IDeviceFeatures : IDispatch
{
	// IModule Features
	[propget, id(1), helpstring("property SupportsMemory")] HRESULT SupportsMemory([out, retval] VARIANT_BOOL* pVal);
	[propget, id(2), helpstring("property SupportsSettings")] HRESULT SupportsSettings([out, retval] VARIANT_BOOL* pVal);
	[propget, id(3), helpstring("property SupportsControl")] HRESULT SupportsControl([out, retval] VARIANT_BOOL* pVal);

	/*
	// IMemory Features
	[propget, id(4), helpstring("property SupportsTrunking")] HRESULT SupportsTrunking([out, retval] VARIANT_BOOL* pVal);

	// ISettings Features
	[propget, id(5), helpstring("property SupportsAttenuate")] HRESULT SupportsAttenuate([out, retval] VARIANT_BOOL* pVal);
	[propget, id(6), helpstring("property SupportsKeyBeep")] HRESULT SupportsKeyBeep([out, retval] VARIANT_BOOL* pVal);
	[propget, id(7), helpstring("property SupportsContrast")] HRESULT SupportsContrast([out, retval] VARIANT_BOOL* pVal);
	[propget, id(8), helpstring("property SupportsAutoLight")] HRESULT SupportsAutoLight([out, retval] VARIANT_BOOL* pVal);

	// IControl Features
	[propget, id(9), helpstring("property SupportsFrequency")] HRESULT SupportsFrequency([out, retval] VARIANT_BOOL* pVal);
	[propget, id(10), helpstring("property SupportsModulation")] HRESULT SupportsModulation([out, retval] VARIANT_BOOL* pVal);
	[propget, id(11), helpstring("property SupportsSignalStrength")] HRESULT SupportsSignalStrength([out, retval] VARIANT_BOOL* pVal);
	[propget, id(12), helpstring("property SupportsSquelch")] HRESULT SupportsSquelch([out, retval] VARIANT_BOOL* pVal);
	*/
};


// IMemory
[
	object,
	uuid("9E010E0E-A95D-40c6-AA47-A69C685CD0B7"),
	dual,
	helpstring("IMemory Interface"),
	pointer_default(unique)
]
__interface IMemory : IDispatch
{
	[propget, id(1), helpstring("Returns the module's memory layout interface.")] HRESULT Layout([out, retval] IMemoryLayout ** pVal);
};


// IMemoryLayout
[
	object,
	uuid("621D192E-73C3-44e1-A141-5988865F5349"),
	dual,
	helpstring("IMemoryLayout Interface"),
	pointer_default(unique)
]
__interface IMemoryLayout : IDispatch
{
	[propget, id(1), helpstring("property FirstChannel")] HRESULT FirstChannel([out, retval] USHORT* pVal);
	[propget, id(2), helpstring("property LastChannel")] HRESULT LastChannel([out, retval] USHORT* pVal);
	[propget, id(3), helpstring("property ChannelCount")] HRESULT ChannelCount([out, retval] USHORT* pVal);

	[propget, id(4), helpstring("property BankCount")] HRESULT BankCount([out, retval] USHORT* pVal);

	[propget, id(5), helpstring("property BankShortName")] HRESULT BankShortName([in] USHORT Bank, [out, retval] BSTR* pVal);
	[propget, id(6), helpstring("property BankName")] HRESULT BankName([in] USHORT Bank, [out, retval] BSTR* pVal);
};


// ISettings
[
	object,
	uuid("EE9903CC-0179-46b1-AB77-82E000F17345"),
	dual,
	helpstring("ISettings Interface"),
	pointer_default(unique)
]
__interface ISettings : IDispatch
{
	[id(1), helpstring("Reset the radio to factory settings.")] HRESULT FullReset(void);

	// Attenuate
	[propget, id(2), helpstring("Gets and sets the status of the attenuate feature.")] HRESULT Attenuate([out, retval] VARIANT_BOOL* pVal);
	[propput, id(2), helpstring("Gets and sets the status of the attenuate feature.")] HRESULT Attenuate([in] VARIANT_BOOL newVal);

	// Key Beep
	[propget, id(3), helpstring("Gets and sets the status of the key beep feature.")] HRESULT KeyBeep([out, retval] VARIANT_BOOL* pVal);
	[propput, id(3), helpstring("Gets and sets the status of the key beep feature.")] HRESULT KeyBeep([in] VARIANT_BOOL newVal);

	// Contrast
	[propget, id(4), helpstring("Gets and sets the status of the contrast.")] HRESULT Contrast([out, retval] USHORT* pVal);
	[propput, id(4), helpstring("Gets and sets the status of the contrast.")] HRESULT Contrast([in] USHORT newVal);

	// Auto Light
	[propget, id(5), helpstring("Gets and sets the status of the auto light feature.")] HRESULT AutoLight([out, retval] VARIANT_BOOL* pVal);
	[propput, id(5), helpstring("Gets and sets the status of the auto light feature.")] HRESULT AutoLight([in] VARIANT_BOOL newVal);

};


// IControl
[
	object,
	uuid("18AACE57-297E-421f-A96B-9CDEEC21CCA4"),
	dual,
	helpstring("IControl Interface"),
	pointer_default(unique)
]
__interface IControl : IDispatch
{
	// Frequency
	[propget, id(1), helpstring("Gets and sets the current frequency in hertz.")] HRESULT Frequency([out, retval] DOUBLE* pVal);
	[propput, id(1), helpstring("Gets and sets the current frequency in hertz.")] HRESULT Frequency([in] DOUBLE newVal);

	// Modulation
	[propget, id(2), helpstring("Gets and sets the current modulation.")] HRESULT Modulation([out, retval] USHORT* pVal);
	[propput, id(2), helpstring("Gets and sets the current modulation.")] HRESULT Modulation([in] USHORT newVal);

	// Signal Strength
	[propget, id(3), helpstring("Gets the current signal strength.")] HRESULT SignalStrength([out, retval] USHORT* pVal);

	// Squelch
	[propget, id(4), helpstring("Gets the status of the squelch.")] HRESULT Squelch([out, retval] VARIANT_BOOL* pVal);
};


// _IProgressEvents
[
	dispinterface,
	uuid("BA7D205D-7B7C-478e-841B-49B2604C5DBD"),
	helpstring("_IProgressEvents Interface")
]
__interface _IProgressEvents
{
	[id(1), helpstring("method OnProgressStart")] HRESULT OnProgressStart([in] ULONG MaximumRange);
	[id(2), helpstring("method OnProgressUpdate")] HRESULT OnProgressUpdate([in] ULONG Position);
	[id(3), helpstring("method OnProgressEnd")] HRESULT OnProgressEnd(void);
};


// _IControlEvents
[
	dispinterface,
	uuid("B2DFA115-1559-468d-863D-D02EC14769BC"),
	helpstring("_IControlEvents Interface")
]
__interface _IControlEvents
{
	[id(1), helpstring("method OnSquelchChange")] HRESULT OnSquelchChange([in] VARIANT_BOOL SquelchOpen);
};
