#include "StdAfx.h"
#include "internaleventsource.h"

CInternalEventSource::CInternalEventSource(void)
{
	m_pPreEventWaitTargets = NULL;
	m_nPreEventWaitTargetCount = 0;

	m_pEvents = NULL;
	m_pEventTargets = NULL;
	m_nEventCount = 0;
}

CInternalEventSource::~CInternalEventSource(void)
{
	if (m_nPreEventWaitTargetCount != 0)
	{
		delete m_pPreEventWaitTargets;
	}

	if (m_nEventCount != 0)
	{
		delete m_pEvents;
		delete m_pEventTargets;
	}
}

HRESULT CInternalEventSource::AddPreEventWaitTarget (CInternalEventTarget *pPreEventWaitTarget)
{
	ATLTRACE ("CSerialModule::AddPreEventWaitTarget\n");

	int nNewPreEventWaitTargetCount = m_nPreEventWaitTargetCount + 1;

	CInternalEventTarget **pNewPreEventWaitTargets = new CInternalEventTarget * [nNewPreEventWaitTargetCount];
	if (pNewPreEventWaitTargets == NULL) return E_OUTOFMEMORY;

	for (int x = 0; x < m_nPreEventWaitTargetCount; x++)
	{
		pNewPreEventWaitTargets [x] = m_pPreEventWaitTargets [x];
	}

	pNewPreEventWaitTargets [m_nEventCount] = pPreEventWaitTarget;

	delete m_pPreEventWaitTargets;

	m_pPreEventWaitTargets = pNewPreEventWaitTargets;
	m_nPreEventWaitTargetCount = nNewPreEventWaitTargetCount;

	return S_OK;
}

HRESULT CInternalEventSource::RemovePreEventWaitTarget (CInternalEventTarget *pPreEventWaitTarget)
{
	ATLTRACE ("CSerialModule::RemovePreEventWaitTarget\n");

	int nNewPreEventWaitTargetCount = m_nPreEventWaitTargetCount - 1;

	CInternalEventTarget **pNewPreEventWaitTargets = new CInternalEventTarget * [nNewPreEventWaitTargetCount];
	if (pNewPreEventWaitTargets == NULL) return E_OUTOFMEMORY;

	int nNewIndex = 0;
	for (int nOldIndex = 0; nOldIndex < m_nPreEventWaitTargetCount; nOldIndex++)
	{
		if (m_pPreEventWaitTargets [nOldIndex] == pPreEventWaitTarget) // is this the one we are removing?
		{
			// yes it is, so skip it
		}
		else
		{
			pNewPreEventWaitTargets [nNewIndex] = m_pPreEventWaitTargets [nNewIndex];
			nNewIndex++;
		}
	}

	delete m_pPreEventWaitTargets;

	m_pPreEventWaitTargets = pNewPreEventWaitTargets;
	m_nPreEventWaitTargetCount = nNewPreEventWaitTargetCount;

	return S_OK;
}

HRESULT CInternalEventSource::AddEvent (HANDLE hEvent, CInternalEventTarget *pInternalEventTarget)
{
	ATLTRACE ("CSerialModule::AddEvent\n");

	int nNewEventCount = m_nEventCount + 1;

	HANDLE *pNewEvents = new HANDLE [nNewEventCount];
	if (pNewEvents == NULL) return E_OUTOFMEMORY;

	CInternalEventTarget **pNewEventTargets = new CInternalEventTarget * [nNewEventCount];
	if (pNewEventTargets == NULL)
	{
		delete pNewEvents;
		return E_OUTOFMEMORY;
	}

	for (int x = 0; x < m_nEventCount; x++)
	{
		pNewEvents [x] = m_pEvents [x];
		pNewEventTargets [x] = m_pEventTargets [x];
	}

	pNewEvents [m_nEventCount] = hEvent;
	pNewEventTargets [m_nEventCount] = pInternalEventTarget;

	delete m_pEvents;
	delete m_pEventTargets;

	m_pEvents = pNewEvents;
	m_pEventTargets = pNewEventTargets;
	m_nEventCount = nNewEventCount;

	return S_OK;
}

HRESULT CInternalEventSource::RemoveEvent (HANDLE hEvent)
{
	ATLTRACE ("CSerialModule::RemoveEvent\n");

	int nNewEventCount = m_nEventCount - 1;

	HANDLE *pNewEvents = new HANDLE [nNewEventCount];
	if (pNewEvents == NULL) return E_OUTOFMEMORY;

	CInternalEventTarget **pNewEventTargets = new CInternalEventTarget * [nNewEventCount];
	if (pNewEventTargets == NULL)
	{
		delete pNewEvents;
		return E_OUTOFMEMORY;
	}

	int nNewIndex = 0;
	for (int nOldIndex = 0; nOldIndex < m_nEventCount; nOldIndex++)
	{
		if (m_pEvents [nOldIndex] == hEvent) // is this the one we are removing?
		{
			// yes it is, so skip it
		}
		else
		{
			pNewEvents [nNewIndex] = m_pEvents [nOldIndex];
			pNewEventTargets [nNewIndex] = m_pEventTargets [nOldIndex];
			nNewIndex++;
		}
	}

	delete m_pEvents;
	delete m_pEventTargets;

	m_pEvents = pNewEvents;
	m_pEventTargets = pNewEventTargets;
	m_nEventCount = nNewEventCount;

	return S_OK;
}

HANDLE *CInternalEventSource::GetEvents (void)
{
	return m_pEvents;
}

int CInternalEventSource::GetEventCount (void)
{
	return m_nEventCount;
}

HRESULT CInternalEventSource::RaisePreEventWait (void)
{
	HRESULT hResult;

	for (int x = 0; x < m_nPreEventWaitTargetCount; x++)
	{
		hResult = m_pPreEventWaitTargets [x]->PreEventWait ();

		if (FAILED (hResult))
		{
			if (hResult != E_NOTIMPL) return hResult;
		}
	}

	return S_OK;
}

HRESULT CInternalEventSource::RaiseEvent (int nEventIndex)
{
	return m_pEventTargets [nEventIndex]->OnEvent (m_pEvents [nEventIndex]);
}
