// SerialLink.h : Declaration of the gdSerialLink

#pragma once
#include "resource.h"       // main symbols

#include "InternalEventTarget.h"

// ISerialLink
[
	object,
	uuid("BD4E55C5-174E-4364-B00C-B43637EA6A2F"),
	dual,
	helpstring("ISerialLink Interface"),
	pointer_default(unique)
]
__interface ISerialLink : IDispatch
{
	[propget, id(1), helpstring("property PortName")] HRESULT PortName([out, retval] BSTR* pVal);
	[propput, id(1), helpstring("property PortName")] HRESULT PortName([in] BSTR newVal);
	[propget, id(2), helpstring("property PortSpeed")] HRESULT PortSpeed([out, retval] LONG* pVal);
	[propput, id(2), helpstring("property PortSpeed")] HRESULT PortSpeed([in] LONG newVal);
	[propget, id(3), helpstring("property PortStopBits")] HRESULT PortStopBits([out, retval] FLOAT* pVal);
	[propput, id(3), helpstring("property PortStopBits")] HRESULT PortStopBits([in] FLOAT newVal);
	[id(4), helpstring("method OpenPort")] HRESULT OpenPort(void);
	[id(5), helpstring("method ClosePort")] HRESULT ClosePort(void);
	[id(6), helpstring("method SendData")] HRESULT SendData([in, size_is (DataLength)] BYTE *Data, [in] LONG DataLength);
	[id(7), helpstring("method SendDataArray")] HRESULT SendDataArray([in, satype (BYTE)] SAFEARRAY *Data);
	[id(8), helpstring("method SendDataString")] HRESULT SendDataString([in] BSTR Data);
};


// _ISerialLinkEvents
[
	dispinterface,
	uuid("54C3ED1A-68A1-47AD-8DC1-E0CF0E2263C4"),
	helpstring("_ISerialLinkEvents Interface")
]
__interface _ISerialLinkEvents
{
	[id(1), helpstring("method OnSendCompleted")] void OnSendCompleted([in] LONG SentLength);
	[id(2), helpstring("method OnReceiveData")] void OnReceiveData([in, size_is (DataLength)] BYTE *Data, [in] LONG DataLength);
	[id(3), helpstring("method OnReceiveDataArray")] void OnReceiveDataArray([in, satype (BYTE)] SAFEARRAY *Data);
	[id(4), helpstring("method OnReceiveDataString")] void OnReceiveDataString([in] BSTR Data);
	[id(5), helpstring("method OnError")] void OnError ([in] INT ErrorCode);
};


// CSerialLink
[
	coclass,
	threading("apartment"),
	event_source("com"),
	vi_progid("gdSerial.SerialLink"),
	progid("gdSerial.SerialLink.1"),
	version(1.0),
	uuid("D699806D-B321-4ED2-A236-5B0F3F1B730F"),
	helpstring("SerialLink Class"),
	default(ISerialLink),
]
class ATL_NO_VTABLE CSerialLink : public CInternalEventTarget, public ISerialLink
{
	public:
		__event __interface _ISerialLinkEvents;

		DECLARE_PROTECT_FINAL_CONSTRUCT()

		HRESULT FinalConstruct();
		void FinalRelease();

		HRESULT CreateEvents (void);
		HRESULT DestroyEvents (void);

		HRESULT PreEventWait (void);
		HRESULT OnEvent (HANDLE);

	public:
		STDMETHOD(get_PortName)(BSTR*);
		STDMETHOD(put_PortName)(BSTR);
		STDMETHOD(get_PortSpeed)(LONG*);
		STDMETHOD(put_PortSpeed)(LONG);
		STDMETHOD(get_PortStopBits)(FLOAT*l);
		STDMETHOD(put_PortStopBits)(FLOAT);
		STDMETHOD(OpenPort)(void);
		STDMETHOD(ClosePort)(void);
		STDMETHOD(SendData)(BYTE *, LONG);
		STDMETHOD(SendDataArray)(SAFEARRAY*);
		STDMETHOD(SendDataString)(BSTR);

	protected:
		HRESULT StartWaitCommEvent (void);
		HRESULT StartWriteFile (BYTE *, LONG);
		HRESULT StartReadFile (DWORD);

		HRESULT OnCommEvent (void);
		HRESULT OnSendEvent (void);
		HRESULT OnReceiveEvent (void);

		BOOL m_bPortOpen;
		HANDLE m_hCommFile;

		HANDLE m_hEventCommEvent;
		HANDLE m_hEventSend;
		HANDLE m_hEventReceive;

		OVERLAPPED m_OverlappedCommEvent;
		OVERLAPPED m_OverlappedSend;
		OVERLAPPED m_OverlappedReceive;

		LPTSTR m_pPortName;
		DWORD m_dwPortSpeed;
		BYTE m_PortStopBits;

		bool m_bWaitCommEventPending;
		bool m_bSendPending;
		bool m_bReceivePending;

		DWORD m_dwCommEvent;

		BYTE *m_pSendBuffer;
		int m_nSendBufferSize;

		BYTE *m_pReceiveBuffer;
		int m_nReceiveBufferSize;

};

